/*
** Module   : CommonFrame.java
** Abstract : super interface for all converted UI frame interfaces.
**
** Copyright (c) 2005-2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- ----------------Description----------------------
** 001 NVS 20051026   @23330 Created initial version.
** 002 GES 20060204   @24263 Added interfaces for higher level methods
**                           (display/update/set/prompt-for) that use the
**                           lower level methods as primitives and provide
**                           a simple external interface for common usage.
** 003 GES 20060208   @24380 Modifications for redirected I/O and using
**                           the UI statements on streams instead of the
**                           terminal.
** 004 GES 20060223   @24683 Final redirected output support.
** 005 GES 20060228   @24776 Support prompt-for editing blocks.
** 006 GES 20060228   @24783 Support for a special behavior of the
**                           screen-value attribute and the input
**                           function.
** 007 GES 20060301   @24799 New feature to expose proper ASSIGN semantic
**                           in the case where the screen-buffer has no
**                           value for a given widget.
** 008 GES 20060317   @25085 Form header support.
** 009 GES 20060322   @25191 Added new signatures for title processing.
** 010 GES 20060401   @25299 Added display sigs for the case where there
**                           are no data containing widgets in a frame.
** 011 GES 20060404   @25345 Moved enable/disable to a frame element
**                           approach.
** 012 GES 20060413   @25513 Added setScreenValue() method.
** 013 SIY 20060414   @25538 Added setVisible() methods.
** 014 SIY 20060415   @25550 Added enableExcept()/disableExcept() for
**                           single widget.
** 015 ECF 20060424   @25673 Added setScrollable() variant which accepts a
**                           logical.
** 016 SIY 20060424   @25681 Added getter/setter for parent and asWidget()
**                           signatures.
** 017 GES 20060426   @25730 Added underline() methods.
** 018 GES 20060427   @25819 Added setStatisticsColumn() method.
** 019 NVS 20060504   @25977 setTitle() calls from the business logic are
**                           now called setDynamicTitle(). The original
**                           set is preserved for use in the frame creati-
**                           on logic.
** 020 GES 20060627   @27602 Added new setDown() signature to provide
**                           fully dynamic down frame control.
** 021 GES 20060705   @27805 Added setColors() method for color stmt
**                           support (moved here from LogicalTerminal).
** 022 GES 20060707   @27905 Overhaul of setDynamicTitle to support a
**                           full callback model.
** 023 GES 20060712   @28003 Moved getFrameId() into this interface.
** 024 GES 20061120   @31293 Updated for support of dynamic row/column
**                           and down.
** 025 NVS 20070118   @31886 EventList class is used instead of WaitList.
** 026 SIY 20070517   @33750 Javadoc cleanups.
** 027 SIY 20070601   @33924 Added display(BrowseWidget).
** 028 CA  20071122   @35950 Added setRetain(int) method definition.
** 029 GES 20080320   @37598 Added FIRST-CHILD and TYPE attributes.
** 030 CA  20080905   @39741 Added missing getter for VISIBLE attribute.
** 031 SIY 20080926   @39966 Added methods which allow implementations
**                           track initialization start/end.
** 032 CA  20080926   @39962 Added new setDynamicTitle variants which
**                           accept a BaseDataType as a parameter for the
**                           title.
** 033 CA  20080930   @39988 Added accessors for FRAME:WIDTH/HEIGHT-CHARS
**                           and FRAME-COL/ROW attributes.
** 034 CA  20081015   @40089 Added method registerAccumulators.
** 035 CA  20090413   @41773 Added accessors for VISIBLE/HIDDEN/SENSITIVE
**                           attributes.
** 036 SVL 20101004          Added support for WIDTH-PIXELS attribute.
** 037 SIY 20101004          Added more methods.
** 038 SIY 20101005          Added 'UNLESS-HIDDEN' versions of enable() and
**                           display() methods.
** 039 SVL 20101006          Added support for HEIGHT, WIDTH, THREE-D
**                           attributes and VALIDATE method.
** 040 ECF 20101006          Added asWidgetHandle method.
** 041 SIY 20101012          Added more variants for view() and hide() methods.
** 042 SIY 20101014          Changed getX/getY signatures.
** 043 SIY 20101029          Changed setX/setY signatures.
** 044 SIY 20110112          Changes related to new color handling scheme.
** 045 CA  20130221          Refactored usage of PRIVATE-DATA and TYPE attribute. Added 
**                           CommonHandleChain support.
** 046 EVL 20130222          Adding UI related attributes and methods.
** 047 CA  20130223          Added support for STREAM-IO, FONT and IN WINDOW clauses in 
**                           frame-phrase.
** 048 CA  20130927          Removed resource support for P2J frames, as they are not the legacy 
**                           frame (in P2J, the legacy frame is FrameWidget).
*/
 
package com.goldencode.p2j.ui;

import com.goldencode.p2j.util.*;

/**
 * This interface is made of commonly used methods that work at the frame
 * level.
 */
public interface CommonFrame
{
   /** No operation currently in process. */
   public static final int NONE = -1;
   
   /** Prompt-for operation currently in process. */
   public static final int PROMPT_FOR = 0;
   
   /** Set operation currently in process. */
   public static final int SET = 1;
   
   /** Update operation currently in process. */
   public static final int UPDATE = 2;
   
   /**
    * Get this frame as instance of <code>GenericWidget</code>.
    * 
    * @return  reference to this frame as <code>GenericWidget</code>
    *          instance.
    */
   public FrameWidget asWidget();
   
   /**
    * Get this frame wrapped in a handle.
    * 
    * @return  handle to this frame.
    */
   public handle asWidgetHandle();
   
   /**
    * Gets the unique numeric ID of this frame.
    *
    * @return  frame ID
    */
   public int getFrameId();   
   
   /**
    * Gets the VISIBLE attribute of the frame.
    *
    * @return  <code>true</code> if the attribute is set to ON.
    */
   public logical isVisible();

   /**
    * Gets the HIDDEN attribute of the frame.
    *
    * @return  <code>true</code> if the attribute is set to ON.
    */
   public logical isHidden();

   /**
    * Gets the SENSITIVE attribute of the frame.
    *
    * @return  <code>true</code> if the attribute is set to ON.
    */
   public logical isSensitive();

   /**
    * Obtain the current value in the screen buffer for the given widget and 
    * assign this into the backing variable. If there is no value in the
    * screen buffer (if it is <code>null</code>) no assignment will occur.
    *
    * @param    data
    *           Represents the variable to assign and the widget for which
    *           the screen value is to be retrieved.
    */
   public void assignScreenValue(FrameElement data);
   
   /**
    * Applies an event to the widget in focus.
    * <p>
    * This method corresponds to these Progress statements:
    * APPLY FRAME
    *
    * @param event
    *        event to be applied
    */
   public void apply(character event);

   /**
    * Applies an event to the widget in focus.
    * <p>
    * This method corresponds to these Progress statements:
    * APPLY FRAME
    *
    * @param event
    *        event to be applied
    */
   public void apply(String event);

   /**
    * Applies an event to the widget in focus.
    * <p>
    * This method corresponds to these Progress statements:
    * APPLY FRAME
    *
    * @param eventCode
    *        event to be applied
    */
   public void apply(integer eventCode);

   /**
    * Applies an event to the widget in focus.
    * <p>
    * This method corresponds to these Progress statements:
    * APPLY FRAME
    *
    * @param eventCode
    *        event to be applied
    */
   public void apply(double eventCode);

   /**
    * Sets batching mode for frame modification.
    *
    * @param  onOff
    *         <code>true</code> sets batching mode
    */
   public void batch(boolean onOff);
      
   /**
    * Performs CHOOSE processing.
    * <p>
    * This method corresponds to these Progress statements:
    * CHOOSE
    *
    * @param  c
    *         CHOOSE statement options
    */
   public void choose(Choose c);
      
   /**
    * Performs CHOOSE processing.
    * <p>
    * This method corresponds to these Progress statements:
    * CHOOSE COLOR
    *
    * @param  c
    *         CHOOSE statement options
    * @param  cs
    *         color specification for use with CHOOSE
    */
   public void choose(Choose c, ColorSpec cs);
      
   /**
    * Clears the data and colors for all fill-in fields of the frame.
    * For down frames, also clears side labels).
    * <p>
    * This method corresponds to these Progress statements:
    * CLEAR
    *
    * @param  noPause
    *         <code>true</code> if NO-PAUSE option was used
    */
   public void clear(boolean noPause);
      
   /**
    * Clears all occurrences for the down frame and resets the current display
    * position to the top of the frame.
    * <p>
    * This method corresponds to these Progress statements:
    * CLEAR ALL
    *
    * @param  noPause
    *         <code>true</code> if NO-PAUSE option was used
    */
   public void clearAll(boolean noPause);
      
   /**
    * Disables input for all field-level widgets of the frame.
    * <p>
    * This method corresponds to these Progress statements:
    * DISABLE ALL
    */
   public void disable();

   /**
    * Disables input for all field-level widgets of the frame, except the
    * specified ones.
    * <p>
    * This method corresponds to these Progress statements:
    * DISABLE ALL EXCEPT
    * 
    * @param xcpt
    *        List of widgets which will not be enabled.
    */
   public void disableExcept(GenericWidget[] xcpt);

   /**
    * Disables input for all field-level widgets of the frame, except the
    * specified one.
    * <p>
    * This method corresponds to these Progress statements:
    * DISABLE ALL EXCEPT
    * 
    * @param widget
    *        Widget which will not be enabled.
    */
   public void disableExcept(GenericWidget widget);
   
   /**
    * Disables input for the given field-level widgets of the frame.
    * <p>
    * This method corresponds to these Progress statements: DISABLE
    * 
    * @param    list
    *           The widgets to disable.
    */
   public void disable(GenericWidget[] list);
   
   /**
    * Disables input for the given field-level widgets of the frame.
    * <p>
    * This method corresponds to these Progress statements: DISABLE
    * 
    * @param    list
    *           The widgets to disable.
    */
   public void disable(FrameElement[] list);
   
   /**
    * Displays a frame with no "data container" fields on the screen.
    * <p>
    * As part of the processing, the frame marked as visible.  The frame will
    * be brought to the top of the Z-order and refreshed.
    * <p>
    * This method corresponds to these Progress statements:
    * DISPLAY
    */
   public void display();

   /**
    * Refresh specified browse widget.
    * <p>
    * This method corresponds to these Progress statements:
    * DISPLAY WITH BROWSE
    *
    * @param    browse
    *           A widget to refresh. 
    */
   public void display(BrowseWidget browse);

   /**
    * Displays (writes) non-data-containing fields from the frame to the
    * given stream.
    * <p>
    * This method corresponds to these Progress statements:
    * DISPLAY
    *
    * @param    out
    *           A stream instance which is open for writing. 
    */
   public void display(Stream out);
   
   /**
    * Displays a list of fields from the frame on the screen.  As part of
    * this operation, the list of data values are copied to the screen
    * buffer.
    * <p>
    * As part of the processing, the frame and each listed widget will be
    * marked as visible.  The frame will be brought to the top of the Z-order
    * and refreshed.
    * <p>
    * Any <code>null</code> entry in the array is silently ignored.
    * <p>
    * This method corresponds to these Progress statements:
    * DISPLAY
    *
    * @param    data
    *           The list of data (and associated widgets) which is to be
    *           displayed.
    */
   public void display(FrameElement[] data);
   
   /**
    * Displays a list of fields from the frame on the screen.  As part of
    * this operation, the list of data values are copied to the screen
    * buffer.
    * <p>
    * As part of the processing, the frame and each listed widget will be
    * marked as visible.  The frame will be brought to the top of the Z-order
    * and refreshed.
    * <p>
    * Any <code>null</code> entry in the array is silently ignored.
    * <p>
    * This method corresponds to these Progress statements:
    * DISPLAY UNLESS-HIDDEN
    *
    * @param    data
    *           The list of data (and associated widgets) which is to be
    *           displayed.
    */
   public void displayUnlessHidden(FrameElement[] data);

   /**
    * Displays (writes) a list of fields from the frame on the given stream.
    * As part of this operation, the list of data values are copied to the
    * screen buffer.
    * <p>
    * Any <code>null</code> entry in the array is silently ignored.
    * <p>
    * This method corresponds to these Progress statements:
    * DISPLAY
    *
    * @param    out
    *           A stream instance which is open for writing. 
    * @param    data
    *           The list of data (and associated widgets) which is to be
    *           displayed.
    */
   public void display(Stream out, FrameElement[] data);
   
   /**
    * Displays a frame with no "data container" fields on the screen.
    * <p>
    * As part of the processing, the frame marked as visible.  The frame will
    * be brought to the top of the Z-order and refreshed.
    * <p>
    * This method corresponds to these Progress statements:
    * DISPLAY
    * 
    * @param    hWin
    *           A handle to the window where the frame is displayed.
    */
   public void display(handle hWin);

   /**
    * Refresh specified browse widget.
    * <p>
    * This method corresponds to these Progress statements:
    * DISPLAY WITH BROWSE
    *
    * @param    browse
    *           A widget to refresh. 
    * @param    hWin
    *           A handle to the window where the frame is displayed.
    */
   public void display(BrowseWidget browse, handle hWin);

   /**
    * Displays (writes) non-data-containing fields from the frame to the
    * given stream.
    * <p>
    * This method corresponds to these Progress statements:
    * DISPLAY
    *
    * @param    out
    *           A stream instance which is open for writing. 
    * @param    hWin
    *           A handle to the window where the frame is displayed.
    */
   public void display(Stream out, handle hWin);
   
   /**
    * Displays a list of fields from the frame on the screen.  As part of
    * this operation, the list of data values are copied to the screen
    * buffer.
    * <p>
    * As part of the processing, the frame and each listed widget will be
    * marked as visible.  The frame will be brought to the top of the Z-order
    * and refreshed.
    * <p>
    * Any <code>null</code> entry in the array is silently ignored.
    * <p>
    * This method corresponds to these Progress statements:
    * DISPLAY
    *
    * @param    data
    *           The list of data (and associated widgets) which is to be
    *           displayed.
    * @param    hWin
    *           A handle to the window where the frame is displayed.
    */
   public void display(FrameElement[] data, handle hWin);
   
   /**
    * Displays a list of fields from the frame on the screen.  As part of
    * this operation, the list of data values are copied to the screen
    * buffer.
    * <p>
    * As part of the processing, the frame and each listed widget will be
    * marked as visible.  The frame will be brought to the top of the Z-order
    * and refreshed.
    * <p>
    * Any <code>null</code> entry in the array is silently ignored.
    * <p>
    * This method corresponds to these Progress statements:
    * DISPLAY UNLESS-HIDDEN
    *
    * @param    data
    *           The list of data (and associated widgets) which is to be
    *           displayed.
    * @param    hWin
    *           A handle to the window where the frame is displayed.
    */
   public void displayUnlessHidden(FrameElement[] data, handle hWin);

   /**
    * Displays (writes) a list of fields from the frame on the given stream.
    * As part of this operation, the list of data values are copied to the
    * screen buffer.
    * <p>
    * Any <code>null</code> entry in the array is silently ignored.
    * <p>
    * This method corresponds to these Progress statements:
    * DISPLAY
    *
    * @param    out
    *           A stream instance which is open for writing. 
    * @param    data
    *           The list of data (and associated widgets) which is to be
    *           displayed.
    * @param    hWin
    *           A handle to the window where the frame is displayed.
    */
   public void display(Stream out, FrameElement[] data, handle hWin);

   /**
    * Positions the cursor on a new line in a down frame.
    * <p>
    * This method corresponds to these Progress statements:
    * DOWN
    */
   public void down();

   /**
    * Positions the cursor on a new line in a down frame.
    * <p>
    * This method corresponds to these Progress statements:
    * DOWN n
    *
    * @param lines
    *        number of iterations of the data to move down
    */
   public void down(double lines);

   /**
    * Positions the cursor on a new line in a down frame.
    * <p>
    * This method corresponds to these Progress statements:
    * DOWN n
    *
    * @param lines
    *        number of iterations of the data to move down
    */
   public void down(NumberType lines);

   /**
    * Writes a single new-line to the given stream.
    * <p>
    * This method corresponds to these Progress statements:
    * DOWN STREAM s
    *
    * @param    out
    *           A stream instance which is open for writing. 
    */
   public void down(Stream out);

   /**
    * Writes <code>lines</code> number of new-lines to the given stream.
    * <p>
    * This method corresponds to these Progress statements:
    * DOWN STREAM s n
    *
    * @param    out
    *           A stream instance which is open for writing. 
    * @param    lines
    *           Number of new-lines to output to the stream.
    */
   public void down(Stream out, double lines);

   /**
    * Writes <code>lines</code> number of new-lines to the given stream.
    * <p>
    * This method corresponds to these Progress statements:
    * DOWN STREAM s n
    *
    * @param    out
    *           A stream instance which is open for writing. 
    * @param    lines
    *           Number of new-lines to output to the stream.
    */
   public void down(Stream out, NumberType lines);

   /**
    * Enables input for all field-level widgets of the frame.
    * <p>
    * This method corresponds to these Progress statements:
    * ENABLE ALL
    */
   public void enable();

   /**
    * Enables input for all field-level widgets of the frame, except the
    * specified ones.
    * <p>
    * This method corresponds to these Progress statements: ENABLE ALL
    * 
    * @param xcpt
    *        List of widgets which will not be enabled.
    */
   public void enableExcept(GenericWidget[] xcpt);


   /**
    * Enables input for all field-level widgets of the frame, except the
    * specified one.
    * <p>
    * This method corresponds to these Progress statements: ENABLE ALL
    * 
    * @param widget
    *        Widget which will not be enabled.
    */
   public void enableExcept(GenericWidget widget);

   
   /**
    * Enables input for the given field-level widgets of the frame.
    * <p>
    * This method corresponds to these Progress statements: ENABLE
    * 
    * @param    list
    *           The widgets to enable.
    */
   public void enable(GenericWidget[] list);
   
   /**
    * Enables input for the given field-level widgets of the frame.
    * <p>
    * This method corresponds to these Progress statements: 
    * ENABLE UNLESS-HIDDEN
    * 
    * @param    list
    *           The widgets to enable.
    */
   public void enableUnlessHidden(GenericWidget[] list);
   
   /**
    * Enables input for the given field-level widgets of the frame.
    * <p>
    * This method corresponds to these Progress statements: ENABLE
    * 
    * @param    list
    *           The widgets to enable.
    */
   public void enable(FrameElement[] list);
   
   /**
    * Enables input for the given field-level widgets of the frame.
    * <p>
    * This method corresponds to these Progress statements: 
    * ENABLE UNLESS-HIDDEN
    * 
    * @param    list
    *           The widgets to enable.
    */
   public void enableUnlessHidden(FrameElement[] list);

   /**
    * Enables input for all field-level widgets of the frame.
    * <p>
    * This method corresponds to these Progress statements:
    * ENABLE ALL
    * 
    * @param    hWin
    *           A handle to the window where the frame is displayed.
    */
   public void enable(handle hWin);

   /**
    * Enables input for all field-level widgets of the frame, except the
    * specified ones.
    * <p>
    * This method corresponds to these Progress statements: ENABLE ALL
    * 
    * @param    xcpt
    *           List of widgets which will not be enabled.
    * @param    hWin
    *           A handle to the window where the frame is displayed.
    */
   public void enableExcept(GenericWidget[] xcpt, handle hWin);

   /**
    * Enables input for all field-level widgets of the frame, except the
    * specified one.
    * <p>
    * This method corresponds to these Progress statements: ENABLE ALL
    * 
    * @param    widget
    *           Widget which will not be enabled.
    * @param    hWin
    *           A handle to the window where the frame is displayed.
    */
   public void enableExcept(GenericWidget widget, handle hWin);
   
   /**
    * Enables input for the given field-level widgets of the frame.
    * <p>
    * This method corresponds to these Progress statements: ENABLE
    * 
    * @param    list
    *           The widgets to enable.
    * @param    hWin
    *           A handle to the window where the frame is displayed.
    */
   public void enable(GenericWidget[] list, handle hWin);
   
   /**
    * Enables input for the given field-level widgets of the frame.
    * <p>
    * This method corresponds to these Progress statements: 
    * ENABLE UNLESS-HIDDEN
    * 
    * @param    list
    *           The widgets to enable.
    * @param    hWin
    *           A handle to the window where the frame is displayed.
    */
   public void enableUnlessHidden(GenericWidget[] list, handle hWin);
   
   /**
    * Enables input for the given field-level widgets of the frame.
    * <p>
    * This method corresponds to these Progress statements: ENABLE
    * 
    * @param    list
    *           The widgets to enable.
    * @param    hWin
    *           A handle to the window where the frame is displayed.
    */
   public void enable(FrameElement[] list, handle hWin);
   
   /**
    * Enables input for the given field-level widgets of the frame.
    * <p>
    * This method corresponds to these Progress statements: 
    * ENABLE UNLESS-HIDDEN
    * 
    * @param    list
    *           The widgets to enable.
    * @param    hWin
    *           A handle to the window where the frame is displayed.
    */
   public void enableUnlessHidden(FrameElement[] list, handle hWin);
   
   /**
    * Obtain the current value in the screen buffer of the backing data for
    * the given widget and return it as a <code>character</code> type. If
    * there is no value in the screen buffer (if it is <code>null</code>
    * which is the representation for an uninitialized value), then return
    * the empty string.
    *
    * @param    widget
    *           The widget for which the value is to be retrieved.
    *
    * @return   The value as a <code>character</code> type or the empty
    *           string if the screen buffer value is uninitialized.
    */
   public character getScreenValue(GenericWidget widget);
   
   /**
    * Set the current value in the screen buffer of the backing data for
    * the given widget.  If the given value is <code>null</code> then this
    * widget will be set to the uninitialized value.
    *
    * @param    widget
    *           The widget for which the value is to be retrieved.
    * @param    value
    *           The new value for the widget, use <code>null</code> to set
    *           the value as uninitialized.
    */
   public void setScreenValue(GenericWidget widget, character value);
   
   /**
    * Makes the frame invisible with the default pause.
    * <p>
    * This method corresponds to these Progress statements:
    * HIDE FRAME
    */
   public void hide();
   
   /**
    * Makes the frame invisible.
    * <p>
    * This method corresponds to these Progress statements:
    * HIDE FRAME
    *
    * @param    noPause
    *           <code>true</code> if NO-PAUSE option was used
    */
   public void hide(boolean noPause);

   /**
    * If the frame is a PAGE-TOP or PAGE-BOTTOM and the stream is paged,
    * the frame is removed from the list of those used for the header or
    * footer.  Otherwise this is a no-operation.
    * <p>
    * This method corresponds to these Progress statements:
    * HIDE STREAM s FRAME
    *
    * @param    out
    *           A stream instance which is open for writing. 
    * @param    noPause
    *           <code>true</code> if NO-PAUSE option was used
    */
   public void hide(Stream out, boolean noPause);

   /**
    * Makes widgets in the frame invisible.
    * 
    * @param    data
    *           Array of widgets to hide.
    */
   public void hide(FrameElement[] data);
   
   /**
    * Makes widgets in the frame invisible.
    * 
    * @param    data
    *           Array of widgets to hide.
    * @param    noPause
    *           <code>true</code> if NO-PAUSE option was used
    */
   public void hide(FrameElement[] data, boolean noPause);
   
   /**
    * Makes the frame invisible with the default pause.
    * <p>
    * This method corresponds to these Progress statements:
    * HIDE FRAME
    * 
    * @param    hWin
    *           A handle to the window where the frame is displayed.
    */
   public void hide(handle hWin);
   
   /**
    * Makes the frame invisible.
    * <p>
    * This method corresponds to these Progress statements:
    * HIDE FRAME
    *
    * @param    hWin
    *           A handle to the window where the frame is displayed.
    * @param    noPause
    *           <code>true</code> if NO-PAUSE option was used
    */
   public void hide(handle hWin, boolean noPause);

   /**
    * If the frame is a PAGE-TOP or PAGE-BOTTOM and the stream is paged,
    * the frame is removed from the list of those used for the header or
    * footer.  Otherwise this is a no-operation.
    * <p>
    * This method corresponds to these Progress statements:
    * HIDE STREAM s FRAME
    *
    * @param    out
    *           A stream instance which is open for writing. 
    * @param    hWin
    *           A handle to the window where the frame is displayed.
    * @param    noPause
    *           <code>true</code> if NO-PAUSE option was used
    */
   public void hide(Stream out, handle hWin, boolean noPause);

   /**
    * Makes widgets in the frame invisible.
    * 
    * @param    data
    *           Array of widgets to hide.
    * @param    hWin
    *           A handle to the window where the frame is displayed.
    */
   public void hide(FrameElement[] data, handle hWin);
   
   /**
    * Makes widgets in the frame invisible.
    * 
    * @param    data
    *           Array of widgets to hide.
    * @param    hWin
    *           A handle to the window where the frame is displayed.
    * @param    noPause
    *           <code>true</code> if NO-PAUSE option was used
    */
   public void hide(FrameElement[] data, handle hWin, boolean noPause);

   /**
    * Specifies the field to be put in focus during the next input operation.
    * <p>
    * This method corresponds to these Progress statements:
    * NEXT-PROMPT
    *
    * @param  field
    *         widget to be put in focus
    */
   public void nextPrompt(GenericWidget field); 

   /**
    * Enables a list of widgets from the frame on the screen, shifts into
    * input blocking mode via {@link #waitFor} and after the user's edits
    * are complete the list of widgets are disabled.  No data is copied to
    * or from the screen buffer.
    * <p>
    * As part of the processing, the frame and each listed widget will be
    * marked as visible.  The frame will be brought to the top of the Z-order
    * and refreshed.
    * <p>
    * Any <code>null</code> entry in the array is silently ignored.
    * <p>
    * If the unnamed input is redirected, this method will redirect its own
    * execution to the stream form.  In such a case, all interactive 
    * processing is bypassed.
    * <p>
    * This method corresponds to these Progress statements:
    * PROMPT-FOR
    *
    * @param    widgets
    *           The list of widgets which is to be edited.  The data portion
    *           of these elements is not used at all.
    */
   public void promptFor(FrameElement[] widgets);

   /**
    * For each entry in the widget array, the input stream will be read and 
    * the widget's associated field in the screen buffer will be set. Data
    * will not be copied to or from the user's variables.
    * <p>
    * Any error during stream reading or data parsing will cause an error
    * condition to be generated.  Reaching the end of the stream will cause
    * an end condition to be generated.
    * <p>
    * Both the input stream and the current default output stream (if output
    * is redirected) are checked.  If both are set to <code>echo</code> mode,
    * then after all stream input is done, the screen buffer's contents will  
    * be echoed to the current output destination using {@link #view}. This
    * will occur whether or not an error occurs during stream processing and
    * data parsing.
    * <p>
    * Any <code>null</code> entry in the array is silently ignored.
    * <p>
    * This method corresponds to these Progress statements:
    * PROMPT-FOR
    *
    * @param    in
    *           A stream instance which is open for reading. 
    * @param    widgets
    *           The list of widgets which are to receive input from the
    *           stream.
    */
   public void promptFor(Stream in, FrameElement[] widgets);
   
   /**
    * Enables a list of widgets from the frame on the screen, shifts into
    * input blocking mode via {@link #waitFor} and after the user's edits
    * are complete the list of widgets are disabled.  No data is copied to
    * or from the screen buffer.
    * <p>
    * As part of the processing, the frame and each listed widget will be
    * marked as visible.  The frame will be brought to the top of the Z-order
    * and refreshed.
    * <p>
    * Any <code>null</code> entry in the array is silently ignored.
    * <p>
    * If the unnamed input is redirected, this method will redirect its own
    * execution to the stream form.  In such a case, all interactive 
    * processing is bypassed.
    * <p>
    * This method corresponds to these Progress statements:
    * PROMPT-FOR
    *
    * @param    widgets
    *           The list of widgets which is to be edited.  The data portion
    *           of these elements is not used at all.
    * @param    wait
    *           The list of events to treat as a <code>GO</code> event.
    */
   public void promptFor(FrameElement[] widgets, EventList wait);
   
   /**
    * Enables a list of widgets from the frame on the screen, shifts into
    * input blocking mode via {@link #waitFor} and after the user's edits
    * are complete the list of widgets are disabled.  No data is copied to
    * or from the screen buffer.
    * <p>
    * As part of the processing, the frame and each listed widget will be
    * marked as visible.  The frame will be brought to the top of the Z-order
    * and refreshed.
    * <p>
    * Any <code>null</code> entry in the array is silently ignored.
    * <p>
    * This method corresponds to these Progress statements:
    * PROMPT-FOR
    *
    * @param    widgets
    *           The list of widgets which is to be edited.
    */
   public void promptFor(GenericWidget[] widgets);

   /**
    * Enables a list of widgets from the frame on the screen, shifts into
    * input blocking mode via {@link #waitFor} and after the user's edits
    * are complete the list of widgets are disabled.  No data is copied to
    * or from the screen buffer.
    * <p>
    * As part of the processing, the frame and each listed widget will be
    * marked as visible.  The frame will be brought to the top of the Z-order
    * and refreshed.
    * <p>
    * Any <code>null</code> entry in the array is silently ignored.
    * <p>
    * This method corresponds to these Progress statements:
    * PROMPT-FOR
    *
    * @param    widgets
    *           The list of widgets which is to be edited.
    * @param    wait
    *           The list of events to treat as a <code>GO</code> event.
    */
   public void promptFor(GenericWidget[] widgets, EventList wait);

   /**
    * Enables a list of widgets from the frame on the screen, shifts into
    * input blocking mode via {@link #waitFor} and after the user's edits
    * are complete the list of widgets are disabled.  No data is copied to
    * or from the screen buffer.
    * <p>
    * As part of the processing, the frame and each listed widget will be
    * marked as visible.  The frame will be brought to the top of the Z-order
    * and refreshed.
    * <p>
    * Any <code>null</code> entry in the array is silently ignored.
    * <p>
    * If the unnamed input is redirected, this method will redirect its own
    * execution to the stream form.  In such a case, all interactive 
    * processing is bypassed.
    * <p>
    * This method corresponds to these Progress statements:
    * PROMPT-FOR
    *
    * @param    widgets
    *           The list of widgets which is to be edited.  The data portion
    *           of these elements is not used at all.
    * @param    hWin
    *           A handle to the window where the frame is displayed.
    */
   public void promptFor(FrameElement[] widgets, handle hWin);

   /**
    * For each entry in the widget array, the input stream will be read and 
    * the widget's associated field in the screen buffer will be set. Data
    * will not be copied to or from the user's variables.
    * <p>
    * Any error during stream reading or data parsing will cause an error
    * condition to be generated.  Reaching the end of the stream will cause
    * an end condition to be generated.
    * <p>
    * Both the input stream and the current default output stream (if output
    * is redirected) are checked.  If both are set to <code>echo</code> mode,
    * then after all stream input is done, the screen buffer's contents will  
    * be echoed to the current output destination using {@link #view}. This
    * will occur whether or not an error occurs during stream processing and
    * data parsing.
    * <p>
    * Any <code>null</code> entry in the array is silently ignored.
    * <p>
    * This method corresponds to these Progress statements:
    * PROMPT-FOR
    *
    * @param    in
    *           A stream instance which is open for reading. 
    * @param    widgets
    *           The list of widgets which are to receive input from the
    *           stream.
    * @param    hWin
    *           A handle to the window where the frame is displayed.
    */
   public void promptFor(Stream in, FrameElement[] widgets, handle hWin);
   
   /**
    * Enables a list of widgets from the frame on the screen, shifts into
    * input blocking mode via {@link #waitFor} and after the user's edits
    * are complete the list of widgets are disabled.  No data is copied to
    * or from the screen buffer.
    * <p>
    * As part of the processing, the frame and each listed widget will be
    * marked as visible.  The frame will be brought to the top of the Z-order
    * and refreshed.
    * <p>
    * Any <code>null</code> entry in the array is silently ignored.
    * <p>
    * If the unnamed input is redirected, this method will redirect its own
    * execution to the stream form.  In such a case, all interactive 
    * processing is bypassed.
    * <p>
    * This method corresponds to these Progress statements:
    * PROMPT-FOR
    *
    * @param    widgets
    *           The list of widgets which is to be edited.  The data portion
    *           of these elements is not used at all.
    * @param    wait
    *           The list of events to treat as a <code>GO</code> event.
    * @param    hWin
    *           A handle to the window where the frame is displayed.
    */
   public void promptFor(FrameElement[] widgets, EventList wait, handle hWin);
   
   /**
    * Enables a list of widgets from the frame on the screen, shifts into
    * input blocking mode via {@link #waitFor} and after the user's edits
    * are complete the list of widgets are disabled.  No data is copied to
    * or from the screen buffer.
    * <p>
    * As part of the processing, the frame and each listed widget will be
    * marked as visible.  The frame will be brought to the top of the Z-order
    * and refreshed.
    * <p>
    * Any <code>null</code> entry in the array is silently ignored.
    * <p>
    * This method corresponds to these Progress statements:
    * PROMPT-FOR
    *
    * @param    widgets
    *           The list of widgets which is to be edited.
    * @param    hWin
    *           A handle to the window where the frame is displayed.
    */
   public void promptFor(GenericWidget[] widgets, handle hWin);

   /**
    * Enables a list of widgets from the frame on the screen, shifts into
    * input blocking mode via {@link #waitFor} and after the user's edits
    * are complete the list of widgets are disabled.  No data is copied to
    * or from the screen buffer.
    * <p>
    * As part of the processing, the frame and each listed widget will be
    * marked as visible.  The frame will be brought to the top of the Z-order
    * and refreshed.
    * <p>
    * Any <code>null</code> entry in the array is silently ignored.
    * <p>
    * This method corresponds to these Progress statements:
    * PROMPT-FOR
    *
    * @param    widgets
    *           The list of widgets which is to be edited.
    * @param    wait
    *           The list of events to treat as a <code>GO</code> event.
    * @param    hWin
    *           A handle to the window where the frame is displayed.
    */
   public void promptFor(GenericWidget[] widgets, EventList wait, handle hWin);
   
   /**
    * Register header frame elements.
    * <p>
    * This method corresponds to these Progress statements:
    * FORM HEADER
    * 
    * @param   fe
    *          Array of headers to register.
    */
   public void registerHeader(FrameElement[] fe);
   
   /**
    * Register all the widgets which need accumulation with this frame.
    * 
    * @param    fe
    *           The list of widgets that may need accumulation.
    */
   public void registerAccumulators(FrameElement[] fe);
   
   /**
    * Scrolls the lines of the down frame up or down.
    * <p>
    * This method corresponds to these Progress statements:
    * SCROLL
    *
    * @param  fromCurrent
    *         <code>true</code> if FROM-CURRENT option was used
    * @param  upDown
    *         <code>true</code> if scrolling up
    */
   public void scroll(boolean fromCurrent, boolean upDown);

   /**
    * Enables a list of widgets, shifts into input blocking mode via 
    * {@link #waitFor}, after the user's edits are complete the list of
    * widgets is disabled and the edited data is copied from the screen 
    * buffer to the listed frame elements. 
    * <p>
    * As part of the processing, the frame and each listed widget will be
    * marked as visible.  The frame will be brought to the top of the Z-order
    * and refreshed.
    * <p>
    * Any <code>null</code> entry in the array is silently ignored.
    * <p>
    * The assignment honors any {@link EmbeddedAssignment} instances in the
    * frame element list, in the order in which they appear.  Otherwise, 
    * these do not affect the user interface processing in any way.
    * <p>
    * If the unnamed input is redirected, this method will redirect its own
    * execution to the stream form.  In such a case, all interactive 
    * processing is bypassed.
    * <p>
    * This method corresponds to these Progress statements:
    * SET
    *
    * @param    data
    *           The list of widgets which is to be edited.  The data portion
    *           of these elements is only used on output (they are assigned
    *           to the values provided by the user).
    */
   public void set(FrameElement[] data);
   
   /**
    * For each entry in the widget array, the input stream will be read and 
    * the widget's associated field in the screen buffer will be set. Data
    * will be copied back from the screen buffer to the user's variables.
    * <p>
    * Any error during stream reading or data parsing will cause an error
    * condition to be generated.  Reaching the end of the stream will cause
    * an end condition to be generated.
    * <p>
    * Both the input stream and the current default output stream (if output
    * is redirected) are checked.  If both are set to <code>echo</code> mode,
    * then after all stream input is done, the screen buffer's contents will  
    * be echoed to the current output destination using {@link #view}. This
    * will occur whether or not an error occurs during stream processing and
    * data parsing.
    * <p>
    * Any <code>null</code> entry in the array is silently ignored.
    * <p>
    * The assignment honors any {@link EmbeddedAssignment} instances in the
    * frame element list, in the order in which they appear.  Otherwise, 
    * these do not affect the user interface processing in any way.
    * <p>
    * This method corresponds to these Progress statements:
    * SET
    *
    * @param    in
    *           A stream instance which is open for reading. 
    * @param    data
    *           The list of widgets which are to receive input from the
    *           stream.
    */
   public void set(Stream in, FrameElement[] data);
   
   /**
    * Enables a list of widgets, shifts into input blocking mode via 
    * {@link #waitFor}, after the user's edits are complete the list of
    * widgets is disabled and the edited data is copied from the screen 
    * buffer to the listed frame elements. 
    * <p>
    * As part of the processing, the frame and each listed widget will be
    * marked as visible.  The frame will be brought to the top of the Z-order
    * and refreshed.
    * <p>
    * Any <code>null</code> entry in the array is silently ignored.
    * <p>
    * The assignment honors any {@link EmbeddedAssignment} instances in the
    * frame element list, in the order in which they appear.  Otherwise, 
    * these do not affect the user interface processing in any way.
    * <p>
    * If the unnamed input is redirected, this method will redirect its own
    * execution to the stream form.  In such a case, all interactive 
    * processing is bypassed.
    * <p>
    * This method corresponds to these Progress statements:
    * SET
    *
    * @param    data
    *           The list of widgets which is to be edited.  The data portion
    *           of these elements is only used on output (they are assigned
    *           to the values provided by the user).
    * @param    wait
    *           The list of events to treat as a <code>GO</code> event.
    */
   public void set(FrameElement[] data, EventList wait);

   /**
    * Enables a list of widgets, shifts into input blocking mode via 
    * {@link #waitFor}, after the user's edits are complete the list of
    * widgets is disabled and the edited data is copied from the screen 
    * buffer to the listed frame elements. 
    * <p>
    * As part of the processing, the frame and each listed widget will be
    * marked as visible.  The frame will be brought to the top of the Z-order
    * and refreshed.
    * <p>
    * Any <code>null</code> entry in the array is silently ignored.
    * <p>
    * The assignment honors any {@link EmbeddedAssignment} instances in the
    * frame element list, in the order in which they appear.  Otherwise, 
    * these do not affect the user interface processing in any way.
    * <p>
    * If the unnamed input is redirected, this method will redirect its own
    * execution to the stream form.  In such a case, all interactive 
    * processing is bypassed.
    * <p>
    * This method corresponds to these Progress statements:
    * SET
    *
    * @param    data
    *           The list of widgets which is to be edited.  The data portion
    *           of these elements is only used on output (they are assigned
    *           to the values provided by the user).
    * @param    hWin
    *           A handle to the window where the frame is displayed.
    */
   public void set(FrameElement[] data, handle hWin);
   
   /**
    * For each entry in the widget array, the input stream will be read and 
    * the widget's associated field in the screen buffer will be set. Data
    * will be copied back from the screen buffer to the user's variables.
    * <p>
    * Any error during stream reading or data parsing will cause an error
    * condition to be generated.  Reaching the end of the stream will cause
    * an end condition to be generated.
    * <p>
    * Both the input stream and the current default output stream (if output
    * is redirected) are checked.  If both are set to <code>echo</code> mode,
    * then after all stream input is done, the screen buffer's contents will  
    * be echoed to the current output destination using {@link #view}. This
    * will occur whether or not an error occurs during stream processing and
    * data parsing.
    * <p>
    * Any <code>null</code> entry in the array is silently ignored.
    * <p>
    * The assignment honors any {@link EmbeddedAssignment} instances in the
    * frame element list, in the order in which they appear.  Otherwise, 
    * these do not affect the user interface processing in any way.
    * <p>
    * This method corresponds to these Progress statements:
    * SET
    *
    * @param    in
    *           A stream instance which is open for reading. 
    * @param    data
    *           The list of widgets which are to receive input from the
    *           stream.
    * @param    hWin
    *           A handle to the window where the frame is displayed.
    */
   public void set(Stream in, FrameElement[] data, handle hWin);
   
   /**
    * Enables a list of widgets, shifts into input blocking mode via 
    * {@link #waitFor}, after the user's edits are complete the list of
    * widgets is disabled and the edited data is copied from the screen 
    * buffer to the listed frame elements. 
    * <p>
    * As part of the processing, the frame and each listed widget will be
    * marked as visible.  The frame will be brought to the top of the Z-order
    * and refreshed.
    * <p>
    * Any <code>null</code> entry in the array is silently ignored.
    * <p>
    * The assignment honors any {@link EmbeddedAssignment} instances in the
    * frame element list, in the order in which they appear.  Otherwise, 
    * these do not affect the user interface processing in any way.
    * <p>
    * If the unnamed input is redirected, this method will redirect its own
    * execution to the stream form.  In such a case, all interactive 
    * processing is bypassed.
    * <p>
    * This method corresponds to these Progress statements:
    * SET
    *
    * @param    data
    *           The list of widgets which is to be edited.  The data portion
    *           of these elements is only used on output (they are assigned
    *           to the values provided by the user).
    * @param    wait
    *           The list of events to treat as a <code>GO</code> event.
    * @param    hWin
    *           A handle to the window where the frame is displayed.
    */
   public void set(FrameElement[] data, EventList wait, handle hWin);
   
   /**
    * Positions the cursor on a new line of the down frame.
    * <p>
    * This method corresponds to these Progress statements:
    * UP
    */
   public void up();

   /**
    * Positions the cursor on a new line of the down frame.
    * <p>
    * This method corresponds to these Progress statements:
    * UP n
    *
    * @param  lines
    *         number of iterations of the data to scroll up (positive) or
    *         down (negative).
    */
   public void up(double lines);

   /**
    * Positions the cursor on a new line of the down frame.
    * <p>
    * This method corresponds to these Progress statements:
    * UP n
    *
    * @param  lines
    *         number of iterations of the data to scroll up (positive) or
    *         down (negative).
   */
   public void up(NumberType lines);

   /**
    * This method exists to support a valid form of the Progress 
    * <code>UP</code> statement which has no effect.
    * <p>
    * This method corresponds to these Progress statements:
    * UP STREAM s
    *
    * @param    out
    *           A stream instance which is open for writing. 
    */
   public void up(Stream out);

   /**
    * If <code>lines</code> is a negative number, the absolute value of that
    * number of new-lines will be written to the given stream.
    * <p>
    * This method corresponds to these Progress statements:
    * UP STREAM s n
    *
    * @param    out
    *           A stream instance which is open for writing. 
    * @param    lines
    *           If negative, the number of new-lines to output to the stream.
    *           Otherwise, this has no effect.
    */
   public void up(Stream out, double lines);

   /**
    * If <code>lines</code> is a negative number, the absolute value of that
    * number of new-lines will be written to the given stream.
    * <p>
    * This method corresponds to these Progress statements:
    * UP STREAM s n
    *
    * @param    out
    *           A stream instance which is open for writing. 
    * @param    lines
    *           If negative, the number of new-lines to output to the stream.
    *           Otherwise, this has no effect.
    */
   public void up(Stream out, NumberType lines);

   /**
    * Copies data from the frame elements to each corresponding widget, then
    * enables the list of widgets, shifts into input blocking mode via
    * {@link #waitFor}, after the user's edits are complete the list of 
    * widgets are disabled and the edited data is copied from the screen 
    * buffer to the listed frame elements.  
    * <p>
    * As part of the processing, the frame and each listed widget will be
    * marked as visible.  The frame will be brought to the top of the Z-order
    * and refreshed.
    * <p>
    * Any <code>null</code> entry in the array is silently ignored.
    * <p>
    * This method uses {@link #display} and {@link #set} to do its real
    * work.  The only difference is that the <code>set</code> is embedded
    * inside a "hidden" <code>TransactionManager</code> block which handles
    * undo/retry on <code>error</code> conditions and undo/leave on 
    * <code>end</code> conditions.
    * <p>
    * The assignment honors any {@link EmbeddedAssignment} instances in the
    * frame element list, in the order in which they appear.  Otherwise, 
    * these do not affect the user interface processing in any way.
    * <p>
    * If the unnamed input is redirected, this method will redirect its own
    * execution to the stream form.  In such a case, all interactive 
    * processing is bypassed.
    * <p>
    * This method corresponds to these Progress statements:
    * UPDATE
    *
    * @param    data
    *           The list of data and widgets which is to be updated.
    */
   public void update(FrameElement[] data);
   
   /**
    * For each entry in the widget array, the input stream will be read and 
    * the widget's associated field in the screen buffer will be set. The 
    * data passed in will be copied from the user's variables into the screen
    * buffer.  After the input is read from the stream into the screen buffer,
    * that data will be copied back from the screen buffer to the user's
    * variables.  
    * <p>
    * Any error during stream reading or data parsing will cause an error
    * condition to be generated.  Reaching the end of the stream will cause
    * an end condition to be generated.  Unlike the <code>update</code>
    * version in this class which has a "hidden" retry block around the
    * core "set" processing, this streams version has no such feature. Errors
    * will be immediately seen by the caller.
    * <p>
    * Both the input stream and the current default output stream (if output
    * is redirected) are checked.  If both are set to <code>echo</code> mode,
    * then after all stream input is done, the screen buffer's contents will  
    * be echoed to the current output destination using {@link #view}. This
    * will occur whether or not an error occurs during stream processing and
    * data parsing.
    * <p>
    * Any <code>null</code> entry in the array is silently ignored.
    * <p>
    * The assignment honors any {@link EmbeddedAssignment} instances in the
    * frame element list, in the order in which they appear.  Otherwise, 
    * these do not affect the user interface processing in any way.
    * <p>
    * This method corresponds to these Progress statements:
    * UPDATE
    *
    * @param    in
    *           A stream instance which is open for reading. 
    * @param    data
    *           The list of widgets which are to receive input from the
    *           stream.
    */
   public void update(Stream in, FrameElement[] data);
   
   /**
    * Copies data from the frame elements to each corresponding widget, then
    * enables the list of widgets, shifts into input blocking mode via
    * {@link #waitFor}, after the user's edits are complete the list of 
    * widgets are disabled and the edited data is copied from the screen 
    * buffer to the listed frame elements.  
    * <p>
    * As part of the processing, the frame and each listed widget will be
    * marked as visible.  The frame will be brought to the top of the Z-order
    * and refreshed.
    * <p>
    * Any <code>null</code> entry in the array is silently ignored.
    * <p>
    * This method uses {@link #display} and {@link #set} to do its real
    * work.  The only difference is that the <code>set</code> is embedded
    * inside a "hidden" <code>TransactionManager</code> block which handles
    * undo/retry on <code>error</code> conditions and undo/leave on 
    * <code>end</code> conditions.
    * <p>
    * The assignment honors any {@link EmbeddedAssignment} instances in the
    * frame element list, in the order in which they appear.  Otherwise, 
    * these do not affect the user interface processing in any way.
    * <p>
    * If the unnamed input is redirected, this method will redirect its own
    * execution to the stream form.  In such a case, all interactive 
    * processing is bypassed.
    * <p>
    * This method corresponds to these Progress statements:
    * UPDATE
    *
    * @param    data
    *           The list of data and widgets which is to be updated.
    * @param    wait
    *           The list of events to treat as a <code>GO</code> event.
    */
   public void update(FrameElement[] data, EventList wait);

   /**
    * Copies data from the frame elements to each corresponding widget, then
    * enables the list of widgets, shifts into input blocking mode via
    * {@link #waitFor}, after the user's edits are complete the list of 
    * widgets are disabled and the edited data is copied from the screen 
    * buffer to the listed frame elements.  
    * <p>
    * As part of the processing, the frame and each listed widget will be
    * marked as visible.  The frame will be brought to the top of the Z-order
    * and refreshed.
    * <p>
    * Any <code>null</code> entry in the array is silently ignored.
    * <p>
    * This method uses {@link #display} and {@link #set} to do its real
    * work.  The only difference is that the <code>set</code> is embedded
    * inside a "hidden" <code>TransactionManager</code> block which handles
    * undo/retry on <code>error</code> conditions and undo/leave on 
    * <code>end</code> conditions.
    * <p>
    * The assignment honors any {@link EmbeddedAssignment} instances in the
    * frame element list, in the order in which they appear.  Otherwise, 
    * these do not affect the user interface processing in any way.
    * <p>
    * If the unnamed input is redirected, this method will redirect its own
    * execution to the stream form.  In such a case, all interactive 
    * processing is bypassed.
    * <p>
    * This method corresponds to these Progress statements:
    * UPDATE
    *
    * @param    data
    *           The list of data and widgets which is to be updated.
    * @param    hWin
    *           A handle to the window where the frame is displayed.
    */
   public void update(FrameElement[] data, handle hWin);
   
   /**
    * For each entry in the widget array, the input stream will be read and 
    * the widget's associated field in the screen buffer will be set. The 
    * data passed in will be copied from the user's variables into the screen
    * buffer.  After the input is read from the stream into the screen buffer,
    * that data will be copied back from the screen buffer to the user's
    * variables.  
    * <p>
    * Any error during stream reading or data parsing will cause an error
    * condition to be generated.  Reaching the end of the stream will cause
    * an end condition to be generated.  Unlike the <code>update</code>
    * version in this class which has a "hidden" retry block around the
    * core "set" processing, this streams version has no such feature. Errors
    * will be immediately seen by the caller.
    * <p>
    * Both the input stream and the current default output stream (if output
    * is redirected) are checked.  If both are set to <code>echo</code> mode,
    * then after all stream input is done, the screen buffer's contents will  
    * be echoed to the current output destination using {@link #view}. This
    * will occur whether or not an error occurs during stream processing and
    * data parsing.
    * <p>
    * Any <code>null</code> entry in the array is silently ignored.
    * <p>
    * The assignment honors any {@link EmbeddedAssignment} instances in the
    * frame element list, in the order in which they appear.  Otherwise, 
    * these do not affect the user interface processing in any way.
    * <p>
    * This method corresponds to these Progress statements:
    * UPDATE
    *
    * @param    in
    *           A stream instance which is open for reading. 
    * @param    data
    *           The list of widgets which are to receive input from the
    *           stream.
    * @param    hWin
    *           A handle to the window where the frame is displayed.
    */
   public void update(Stream in, FrameElement[] data, handle hWin);
   
   /**
    * Copies data from the frame elements to each corresponding widget, then
    * enables the list of widgets, shifts into input blocking mode via
    * {@link #waitFor}, after the user's edits are complete the list of 
    * widgets are disabled and the edited data is copied from the screen 
    * buffer to the listed frame elements.  
    * <p>
    * As part of the processing, the frame and each listed widget will be
    * marked as visible.  The frame will be brought to the top of the Z-order
    * and refreshed.
    * <p>
    * Any <code>null</code> entry in the array is silently ignored.
    * <p>
    * This method uses {@link #display} and {@link #set} to do its real
    * work.  The only difference is that the <code>set</code> is embedded
    * inside a "hidden" <code>TransactionManager</code> block which handles
    * undo/retry on <code>error</code> conditions and undo/leave on 
    * <code>end</code> conditions.
    * <p>
    * The assignment honors any {@link EmbeddedAssignment} instances in the
    * frame element list, in the order in which they appear.  Otherwise, 
    * these do not affect the user interface processing in any way.
    * <p>
    * If the unnamed input is redirected, this method will redirect its own
    * execution to the stream form.  In such a case, all interactive 
    * processing is bypassed.
    * <p>
    * This method corresponds to these Progress statements:
    * UPDATE
    *
    * @param    data
    *           The list of data and widgets which is to be updated.
    * @param    wait
    *           The list of events to treat as a <code>GO</code> event.
    * @param    hWin
    *           A handle to the window where the frame is displayed.
    */
   public void update(FrameElement[] data, EventList wait, handle hWin);
      
   /**
    * Makes the frame visible.
    * <p>
    * This method corresponds to these Progress statements:
    * VIEW FRAME
    */
   public void view();

   /**
    * Makes widgets in frame visible.
    * 
    * @param    list
    *           The list of data and widgets which is to be viewed.
    */
   public void view(FrameElement[] list);
   
   /**
    * Writes the current screen-buffer contents to the given stream.
    * <p>
    * This method corresponds to these Progress statements:
    * VIEW STREAM s FRAME
    *
    * @param    out
    *           A stream instance which is open for writing. 
    */
   public void view(Stream out);
   
   /**
    * Makes the frame visible.
    * <p>
    * This method corresponds to these Progress statements:
    * VIEW FRAME
    * 
    * @param    hWin
    *           A handle to the window where the frame is displayed.
    */
   public void view(handle hWin);
   
   /**
    * Makes widgets in frame visible.
    * 
    * @param    list
    *           The list of data and widgets which is to be viewed.
    * @param    hWin
    *           A handle to the window where the frame is displayed.
    */
   public void view(FrameElement[] list, handle hWin);
   
   /**
    * Writes the current screen-buffer contents to the given stream.
    * <p>
    * This method corresponds to these Progress statements:
    * VIEW STREAM s FRAME
    *
    * @param    out
    *           A stream instance which is open for writing. 
    * @param    hWin
    *           A handle to the window where the frame is displayed.
    */
   public void view(Stream out, handle hWin);
   
   /**
    * Returns the column position of the left corner of the frame within its
    * window.
    * <p>
    * This method corresponds to the Progress built-in function
    * FRAME-COL
    * 
    * @return value of the attribute.
    */
   public decimal frameCol();

   /**
    * Returns the number of iterations in a down frame.
    * <p>
    * This method corresponds to the Progress built-in function
    * FRAME-DOWN
    * 
    * @return value of the attribute.
    */
   public integer frameDown();

   /**
    * Returns the current logical line number in a down frame.
    * <p>
    * This method corresponds to the Progress built-in function
    * FRAME-LINE
    * 
    * @return value of the attribute.
    */
   public integer frameLine();

   /**
    * Returns the row position of the upper-left corner of the frame within 
    * its window.
    * <p>
    * This method corresponds to the Progress built-in function
    * FRAME-ROW
    * 
    * @return value of the attribute.
    */
   public decimal frameRow();

   /**
    * Sets the ATTR-SPACE option.
    *
    * @param attr_space
    *        <code>true</code> if the option is set to ON.
    */
   public void setAttrSpace(boolean attr_space);
   
   /**
    * Sets the CENTERED option.
    *
    * @param centered
    *        <code>true</code> if the option is set to ON.
    */
   public void setCentered(boolean centered);

   /**
    * Sets custom colors for output fields, input fields, or both in
    * all specified widgets.
    * <p>
    * This method corresponds to these Progress statements: <br>
    * <b>COLOR DISPLAY [PROMPT]</b> <br>
    * <b>COLOR PROMPT</b>
    * 
    * @param    display
    *           A color specification for regular output or <code>null</code>
    *           if no change is required.
    * @param    prompt
    *           A color specification for input or <code>null</code> if
    *           no change is required.
    * @param    widgets
    *           Array of target widgets.  All elements must be in the same
    *           frame.
    */
   public void setColors(ColorSpec       display,
                         ColorSpec       prompt, 
                         GenericWidget[] widgets);
                                
   /**
    * Sets custom colors for the given output field, input field, or both.
    * <p>
    * This method corresponds to these Progress statements: <br>
    * <b>COLOR DISPLAY [PROMPT]</b> <br>
    * <b>COLOR PROMPT</b>
    * 
    * @param    display
    *           A color specification for regular output or <code>null</code>
    *           if no change is required.
    * @param    prompt
    *           A color specification for input or <code>null</code> if
    *           no change is required.
    * @param    widget
    *           The target widget.
    */
   public void setColors(ColorSpec     display,
                         ColorSpec     prompt, 
                         GenericWidget widget);
                                
   /**
    * Sets the display color.
    *
    * @param color
    *        color number
    */
   public void setDcolor(Color color);

   /**
    * Sets the input (prompt-for) color.
    *
    * @param color
    *        color number
    */
   public void setPfcolor(Color color);

   /**
    * Sets the display color.
    *
    * @param color
    *        color number
    */
   public void setDcolor(ColorSpec color);

   /**
    * Sets the input (prompt-for) color.
    *
    * @param color
    *        color number
    */
   public void setPfcolor(ColorSpec color);

   /**
    * Sets the frame column as a callback to read the current value
    * calculated by the given expression. This value is honored at the time
    * that the frame is made visible (for the first time or after being
    * hidden).
    * 
    * @param    column
    *           The expression to evaluate for the frame column.
    */
   public void setColumn(Resolvable column);
   
   /**
    * Sets the frame column as a callback to read the current value stored in
    * the given <code>Accessor</code>. This value is honored at the time that
    * the frame is made visible (for the first time or after being hidden).
    * 
    * @param    column
    *           The instance variable to read for the frame column.
    */
   public void setColumn(Accessor column);
   
   /**
    * Sets the column position for this frame (statically).
    *
    * @param    column
    *           The 1-based column number at which to start displaying the 
    *           frame.  This is converted to a 0-based column number for the
    *           internal runtime.
    */
   public void setColumn(NumberType column);

   /**
    * Sets the column position for this frame (statically).
    *
    * @param    column
    *           The 1-based column number at which to start displaying the 
    *           frame.  This is converted to a 0-based column number for the
    *           internal runtime.
    */
   public void setColumn(double column);

   /**
    * Returns the column position of the upper-left corner of the frame
    * within its window (in coordinates that are relative to the containing
    * window).
    * <p>
    * This method corresponds to the Progress <code>FRAME:COLUMN</code> 
    * attribute.
    * 
    * @return   The column position for the upper-left corner of the frame.
    */
   public integer getColumn();

   /**
    * Returns the row position of the upper-left corner of the frame
    * within its window (in coordinates that are relative to the containing
    * window).
    * <p>
    * This method corresponds to the Progress <code>FRAME:ROW</code>
    * attribute.
    * 
    * @return   The row position for the upper-left corner of the frame.
    */
   public integer getRow();

   /**
    * Sets the number of columns.
    *
    * @param columns
    *        number of columns
    */
   public void setColumns(int columns);

   /**
    * Sets the VIEW-AS DIALOG-BOX option.
    *
    * @param dialogBox
    *        <code>true</code> if the option is set to ON.
    */
   public void setDialogBox(boolean dialogBox);

   /**
    * Sets the size of a DOWN frame as a callback to read the current value
    * calculated by the given expression.  This value is then honored at the
    * time that the frame is made visible (for the first time or after being
    * hidden).
    *
    * @param    down
    *           The expression to evaluate for the number of data iterations
    *           in the current (DOWN) frame.
    */
   public void setDown(Resolvable down);
   
   /**
    * Sets the size of a DOWN frame as a callback to read the current value
    * stored in the given <code>Accessor</code>.  This value is then honored
    * at the time that the frame is made visible (for the first time or after
    * being hidden).
    *
    * @param    down
    *           The simple instance variable to read for the number of data
    *           iterations in the current (DOWN) frame.
    */
   public void setDown(Accessor down);
   
   /**
    * Sets the size of a DOWN frame (statically).
    *
    * @param    down
    *           The number of data iterations in the DOWN frame.
    */
   public void setDown(NumberType down);

   /**
    * Sets the size of a DOWN frame (statically).
    *
    * @param    down
    *           The number of data iterations in the DOWN frame.
    */
   public void setDown(double down);

   /**
    * Sets the height of the widget.
    *
    * @param height
    *        height of the widget
    */
   public void setHeight(int height);

   /**
    * Sets the height of the widget.
    *
    * @param   height
    *          height of the widget
    */
   public void setHeight(integer height);

   /**
    * Sets the height of the widget.
    *
    * @param height
    *        height of the widget
    */
   public void setHeight(double height);

   /**
    * Sets the height of the widget.
    *
    * @param   height
    *          height of the widget
    */
   public void setHeight(decimal height);

   /**
    * Returns the height of the widget.
    *
    * @return  height of the widget, retrieved from the 
    *          <code>FRAME:HEIGHT-CHARS</code> attribute.
    */
   public integer getHeight();

   /**
    * Sets the KEEP-TAB-ORDER option.
    *
    * @param keepTabOrder
    *        <code>true</code> if the option is set to ON.
    */
   public void setKeepTabOrder(boolean keepTabOrder);

   /**
    * Sets the THREE-D option.
    *
    * @param threeD
    *        <code>true</code> if the option is set to ON.
    */
   public void setThreeD(boolean threeD);

   /**
    * Sets the NO-BOX option.
    *
    * @param noBox
    *        <code>true</code> if the option is set to ON.
    */
   public void setNoBox(boolean noBox);

   /**
    * Sets the NO-HELP option.
    *
    * @param noHelp
    *        <code>true</code> if the option is set to ON.
    */
   public void setNoHelp(boolean noHelp);

   /**
    * Sets the NO-HIDE option.
    *
    * @param noHide
    *        <code>true</code> if the option is set to ON.
    */
   public void setNoHide(boolean noHide);

   /**
    * Sets the NO-LABELS option.
    *
    * @param noLabels
    *        <code>true</code> if the option is set to ON.
    */
   public void setNoLabels(boolean noLabels);

   /**
    * Sets the NO-UNDERLINE option.
    *
    * @param noUnderline
    *        <code>true</code> if the option is set to ON.
    */
   public void setNoUnderline(boolean noUnderline);

   /**
    * Sets the NO-VALIDATE option.
    *
    * @param noValidate
    *        <code>true</code> if the option is set to ON.
    */
   public void setNoValidate(boolean noValidate);

   /**
    * Sets the OVERLAY option.
    *
    * @param overlay
    *        <code>true</code> if the option is set to ON.
    */
   public void setOverlay(boolean overlay);

   /**
    * Sets the PAGE-BOTTOM option.
    *
    * @param pageBottom
    *        <code>true</code> if the option is set to ON.
    */
   public void setPageBottom(boolean pageBottom);

   /**
    * Sets the PAGE-TOP option.
    *
    * @param pageTop
    *        <code>true</code> if the option is set to ON.
    */
   public void setPageTop(boolean pageTop);

   /**
    * Sets the PARENT option.
    * 
    * @param   widget
    *          New parent widget for frame.
    */
   public void setParent(GenericWidget widget);

   /**
    * Get the PARENT option.
    * 
    * @return  handle object for the frame handle.
    * 
    */
   public handle getParent();
   
   /**
    * Get the first child of the frame which will be the field group that
    * contains all non-header widgets.
    *
    * @return   The first field group.
    */
   public GenericWidget getFirstChild();
   
   /**
    * Sets the frame retain size.
    *
    * @param retain
    *        retain size
    */
   public void setRetain(int retain);

   /**
    * Sets the frame row as a callback to read the current value calculated
    * by the given expression. This value is honored at the time that the
    * frame is made visible (for the first time or after being hidden).
    * 
    * @param    row
    *           The expression to evaluate for the frame row.
    */
   public void setRow(Resolvable row);
   
   /**
    * Sets the frame row as a callback to read the current value stored in the
    * given <code>Accessor</code>. This value is honored at the time that the
    * frame is made visible (for the first time or after being hidden).
    * 
    * @param    row
    *           The instance variable to read for the frame row.
    */
   public void setRow(Accessor row);
   
   /**
    * Sets the row position for this frame (statically).
    * 
    * @param    row
    *           The 1-based row number at which to start displaying the frame.
    *           This is converted to a 0-based row number for the internal
    *           runtime.
    */
   public void setRow(NumberType row);

   /**
    * Sets the row position for this frame (statically).
    * 
    * @param    row
    *           The 1-based row number at which to start displaying the frame.
    *           This is converted to a 0-based row number for the internal
    *           runtime.
    */
   public void setRow(double row);

   /**
    * Sets the frame scroll size.
    *
    * @param scroll
    *        scroll size
    */
   public void setScroll(int scroll);

   /**
    * Sets the SCROLLABLE option.
    *
    * @param scrollable
    *        <code>true</code> if the option is set to ON.
    */
   public void setScrollable(boolean scrollable);

   /**
    * Sets the SCROLLABLE option.
    *
    * @param scrollable
    *        <code>true</code> if the option is set to ON.
    */
   public void setScrollable(logical scrollable);
   
   /**
    * Sets the SIDE-LABELS option.
    *
    * @param sideLabels
    *        <code>true</code> if the option is set to ON.
    */
   public void setSideLabels(boolean sideLabels);

   /**
    * Sets the frame title.
    *
    * @param    title
    *           The title text to use.
    */
   public void setTitle(character title);

   /**
    * Sets the frame title.
    *
    * @param    title
    *           The title text to use.
    */
   public void setTitle(String title);

   /**
    * Sets the frame title and title color.
    *
    * @param    title
    *           The title text to use.
    * @param    titleColor
    *           The overridden color for the title.
    */
   public void setTitle(character title, ColorSpec titleColor);

   /**
    * Sets the frame title and title color.
    *
    * @param    title
    *           The title text to use.
    * @param    titleColor
    *           The overridden color for the title.
    */
   public void setTitle(String title, ColorSpec titleColor);

   /**
    * Sets the frame title to be used when the frame is viewed for the first
    * time or when the frame is viewed for the first time after being hidden.
    * <p>
    * The title's value will be read dynamically (as a callback) using the
    * given parameter.  The value at the instant of this call is not the
    * value that will necessarily be seen when the frame is viewed.
    *
    * @param    title
    *           The simple variable or field reference that contains the
    *           title text to be used.
    */
   public void setDynamicTitle(Accessor title);

   /**
    * Sets the frame title to be used when the frame is viewed for the first
    * time or when the frame is viewed for the first time after being hidden.
    * <p>
    * The title's value will be read dynamically (as a callback to the given
    * parameter).  The value at the instant of this call is not the value that 
    * will necessarily be seen when the frame is viewed.
    *
    * @param    title
    *           The simple variable or field reference that contains the
    *           title text to be used.
    */
   public void setDynamicTitle(BaseDataType title);

   /**
    * Sets the frame title to be used when the frame is viewed for the first
    * time or when the frame is viewed for the first time after being hidden.
    * <p>
    * The title's value will be read dynamically (as a callback) using the
    * given parameter.  The value at the instant of this call is not the
    * value that will necessarily be seen when the frame is viewed.
    *
    * @param    title
    *           The expression callback that generates the title text to be
    *           used.
    */
   public void setDynamicTitle(CharacterExpression title);

   /**
    * Sets the frame title to be used when the frame is viewed for the first
    * time or when the frame is viewed for the first time after being hidden.
    * <p>
    * The title's value will be read dynamically (as a callback to the given
    * parameter).  The value at the instant of this call is not the value that 
    * will necessarily be seen when the frame is viewed.
    *
    * @param    title
    *           The simple variable or field reference that contains the
    *           title text to be used.
    * @param    titleColor
    *           The overridden color for the title.
    */
   public void setDynamicTitle(BaseDataType title, ColorSpec titleColor);

   /**
    * Sets the frame title to be used when the frame is viewed for the first
    * time or when the frame is viewed for the first time after being hidden.
    * <p>
    * The title's value will be read dynamically (as a callback) using the
    * given parameter.  The value at the instant of this call is not the
    * value that will necessarily be seen when the frame is viewed.
    *
    * @param    title
    *           The simple variable or field reference that contains the
    *           title text to be used.
    * @param    titleColor
    *           The overridden color for the title.
    */
   public void setDynamicTitle(Accessor title, ColorSpec titleColor);

   /**
    * Sets the frame title to be used when the frame is viewed for the first
    * time or when the frame is viewed for the first time after being hidden.
    * <p>
    * The title's value will be read dynamically (as a callback) using the
    * given parameter.  The value at the instant of this call is not the
    * value that will necessarily be seen when the frame is viewed.
    *
    * @param    title
    *           The expression callback that generates the title text to be
    *           used.
    * @param    titleColor
    *           The overridden color for the title.
    */
   public void setDynamicTitle(CharacterExpression title,
                               ColorSpec           titleColor);

   /**
    * Sets the containing frame instance.
    * 
    * @param    hFrame
    *           handle which is expected to contain a <code>FrameWidget</code>
    *           instance
    */
   public void setFrame(handle hFrame);
   
   /**
    * Sets the color for the frame title.
    *
    * @param    titleColor
    *           The overridden color for the title.
    */
   public void setTitleColor(ColorSpec titleColor);

   /**
    * Sets the TOP-ONLY option.
    *
    * @param topOnly
    *        <code>true</code> if the option is set to ON.
    */
   public void setTopOnly(boolean topOnly);
   
   /**
    * Sets the VISIBLE option.
    *
    * @param visible
    *        <code>true</code> if the option is set to ON.
    */
   public void setVisible(boolean visible);

   /**
    * Sets the VISIBLE option.
    *
    * @param visible
    *        <code>true</code> if the option is set to ON.
    */
   public void setVisible(logical visible);
   
   /**
    * Sets new value of SENSITIVE attribute.
    *
    * @param s
    *        new value of SENSITIVE attribute
    */
   public void setSensitive(logical s);

   /**
    * Sets new value of SENSITIVE attribute.
    *
    * @param s
    *        new value of SENSITIVE attribute
    */
   public void setSensitive(boolean s);

   /**
    * Sets the HIDDEN writable attribute.
    *
    * @param    hidden
    *           The new value for the HIDDEN attribute.
    */
   public void setHidden(logical hidden); 

   /**
    * Sets the HIDDEN writable attribute.
    *
    * @param    hidden
    *           The new value for the HIDDEN attribute.
    */
   public void setHidden(boolean hidden); 

   /**
    * Sets the width of the widget.
    *
    * @param    width
    *           width of the widget
    */
   public void setWidth(int width);

   /**
    * Sets the width of the widget.
    *
    * @param    width
    *           width of the widget
    */
   public void setWidth(integer width);

   /**
    * Sets the width of the widget.
    *
    * @param    width
    *           width of the widget
    */
   public void setWidth(double width);

   /**
    * Sets the width of the widget.
    *
    * @param    width
    *           width of the widget
    */
   public void setWidth(decimal width);

   /**
    * Returns the width of the widget.
    *
    * @return  width of the widget, retrieved from the 
    *          <code>FRAME:WIDTH-CHARS</code> attribute.
    */
   public integer getWidth();

   /**
    * Returns the width of the widget in pixels. Corresponds Progress
    * WIDTH-PIXELS attribute.
    *
    * @return The width of the widget in pixels.
    */
   public integer getWidthPixels();

   /**
    * Returns the horizontal position of the frame. Corresponds Progress X
    * attribute.
    *
    * @return the horizontal position of the frame.
    */
   public NumberType getX();

   /**
    * Returns the vertical position of the frame. Corresponds Progress Y
    * attribute.
    *
    * @return the vertical position of the frame.
    */
   public integer getY();

   /**
    * Sets the horizontal position of the frame (Progress X
    * attribute).
    *
    * @param    x
    *           horizontal position to be set.
    */
   public void setX(NumberType x);

   /**
    * Sets the vertical position of the frame (Progress Y
    * attribute).
    *
    * @param    y
    *           vertical position to be set.
    */
   public void setY(NumberType y);

   /**
    * Sets new value of WIDTH-PIXELS attribute.
    *
    * @param    widthPixels
    *           new value of WIDTH-PIXELS attribute
    */
   public void setWidthPixels(integer widthPixels);

   /**
    * Sets the default button for this frame.
    *
    * @param    defaultButton
    *           button widget that works as the default
    */
   public void setDefaultButton(ButtonWidget defaultButton);

    /**
    * Sets the widget which will be used to display the description for 
    * any given statistic that is being automatically aggregated by the
    * {@link #display} processing.
    *
    * @param    widget
    *           The widget to use as the statistics column.
    */
   public void setStatisticsColumn(GenericWidget widget);
   
  /**
    * Notifies the frame about entering a new block scope.
    * <p>
    * This method should use the notification to register a callback
    * for the block exit event. The callback should finalize the frame and
    * all contained widgets.
    */
   public void openScope();

   /**
    * Marks the beginning of the editing block.
    *
    * @param    list
    *           List of frame elements controlled by this editing block. This
    *           is used to ensure that the proper semantics of update or set
    *           are honored at editing block start (optionally copying data
    *           into the screen buffer, enabling widgets and view).
    * @param    type
    *           Specifies if this editing block must operates like
    *           {@link #promptFor} (no copying to or from the screen buffer),
    *           {@link #set} (no copying to the screen buffer) or 
    *           {@link #update} (copying both directions).  Use the constants
    *           PROMPT_FOR, SET and UPDATE to specify this parameter. 
    */
   public void startEditingMode(FrameElement[] list, int type);
   
   /**
    * Marks the beginning of the editing block with caller control over the
    * exit conditions for the loop.
    *
    * @param    list
    *           List of frame elements controlled by this editing block. This
    *           is used to ensure that the proper semantics of update or set
    *           are honored at editing block start (optionally copying data
    *           into the screen buffer, enabling widgets and view).
    * @param    type
    *           Specifies if this editing block must operates like
    *           {@link #promptFor} (no copying to or from the screen buffer),
    *           {@link #set} (no copying to the screen buffer) or 
    *           {@link #update} (copying both directions).  Use the constants
    *           PROMPT_FOR, SET and UPDATE to specify this parameter. 
    * @param    wait
    *           List of events to wait for.
    *
    */
   public void startEditingMode(FrameElement[] list, 
                                int            type,
                                EventList      wait);
   
   /**
    * Displays an underline for the specified field-level widgets of the
    * frame. Does not permanently change the attributes for the given
    * widgets!
    * <p>
    * This method corresponds to these Progress statements:
    * UNDERLINE
    * 
    * @param    list 
    *           List of widgets.
    */
   public void underline(GenericWidget[] list);
   
   /**
    * Displays an underline for the specified field-level widgets of the
    * frame. Does not permanently change the attributes for the given
    * widgets!
    * <p>
    * This method corresponds to these Progress statements:
    * UNDERLINE
    * 
    * @param    list 
    *           List of widget ids.
    */
   public void underline(int[] list);
   
   /**
    * Displays an underline for the specified field-level widgets of the
    * frame. Does not permanently change the attributes for the given
    * widgets!
    * <p>
    * This method corresponds to these Progress statements:
    * UNDERLINE
    * 
    * @param    stream
    *           An explicit stream which will be used for the output.
    * @param    list 
    *           List of widgets.
    */
   public void underline(Stream stream, GenericWidget[] list);
   
   /**
    * Displays an underline for the specified field-level widgets of the
    * frame. Does not permanently change the attributes for the given
    * widgets!
    * <p>
    * This method corresponds to these Progress statements:
    * UNDERLINE
    * 
    * @param    stream
    *           An explicit stream which will be used for the output.
    * @param    list 
    *           List of widget ids.
    */
   public void underline(Stream stream, int[] list);
   
  /**
   * Waits for a default list of events.
   * <p>
   * This method corresponds to the Progress statement: <br>
   * <b>WAIT-FOR </b>
   */
   public void waitFor();

  /**
   * Waits for specified list of events.
   * <p>
   * This method corresponds to the Progress statement: <br>
   * <b>WAIT-FOR </b>
   * 
   * @param   el
   *          List of events to wait for.
   */
   public void waitFor(EventList el);

   /**
    * Waits for specified list of events.
    * <p>
    * This method corresponds to the Progress statement: <br>
    * <b>WAIT-FOR FOCUS widget </b>
    * 
    * @param   el
    *          List of events to wait for.
    * @param   focusWidget
    *          Widget which is in focus.
    */
   public void waitFor(EventList el, GenericWidget focusWidget);

   /**
    * Waits for specified list of events.
    * <p>
    * This method corresponds to the Progress statement: <br>
    * <b>WAIT-FOR PAUSE n </b>
    * 
    * @param   el
    *          List of events to wait for.
    * @param   seconds
    *          Time to wait for event.
    */
   public void waitFor(EventList el, int seconds);

   /**
    * Waits for specified list of events.
    * <p>
    * This method corresponds to the Progress statement: <br>
    * <b>WAIT-FOR FOCUS widget PAUSE n </b>
    * 
    * @param   el
    *          List of events to wait for.
    * @param   focusWidget
    *          Widget which is in focus.
    * @param   seconds
    *          Time to wait for event.
    */
   public void waitFor(EventList el, GenericWidget focusWidget, 
                       int seconds);

   /**
    * Conditionally blocks until a next UI event is available.
    */
   public void waitForNextKey();
   
   /**
    * Checks to see whether a pending GO event is about to end the editing
    * block.
    *
    * @param    list
    *           List of frame elements controlled by this editing block. This
    *           is used to ensure that the proper semantics of update or set
    *           are honored at editing block exit (disabling widgets and
    *           assigning data back).
    *
    * @return   <code>true</code> if there is a pending GO.
    */
   public boolean continueEditing(FrameElement[] list);
   
   /**
    * Conversion of CURRENT-ITERATION attribute (KW_CUR_ITER) of frame widget.
    *
    * Getter of CURRENT-ITERATION attribute.
    *
    * @return current value of CURRENT-ITERATION attribute.
    */
   public integer currentIteration();
   
   /**
    * This method will be called by run-time when frame initialization is
    * started and when frame initialization is finished.
    * 
    * @param   flag
    *          <code>true</code> means start of initialization and
    *          <code>false</code> - end of initialization.
    */
   public void setInsideSetup(boolean flag);

   /**
    * This method implements MOVE-AFTER-TAB function.
    * 
    * @param   widget
    *          Widget reference.
    *
    * @return  <code>true</code> if operation was successful.
    */
   public boolean moveAfterTab(GenericWidget widget);

   /**
    * Executes the validation tests for every supported field-level widget
    * in the frame (except the browse, which you must validate explicitly).
    * Corresponds Progress VALIDATE method.
    *
    * @return  <code>true</code> if validation was successful.
    */
   public logical validate();
   
   /**
    * Gets the integer handle value of the window containing the widget.
    * 
    * @return   The integer handle value.
    */
   public integer getHWND();
   
   /**
    * Getting the FIRST-TAB-ITEM attribute.
    * 
    * @return  nothing.
    */
   public handle getFirstTabItem();

   /**
    * Setting the FIRST-TAB-ITEM attribute.
    * 
    * @param   h
    *          Not used.
    */
   public void setFirstTabItem(handle h);

   /**
    * Getting the NEXT-TAB-ITEM attribute.
    * 
    * @return  nothing.
    */
   public handle getNextTabItem();

   /**
    * Assigns the current widget to the tab position before the specified in the same field group.
    *
    * @param   widget
    *          The handle of the widget before whose we need to make new assignment.
    *
    * @return  <code>true</code> if operation was successful.
    */
   public logical moveBeforeTabItem(GenericWidget widget);

   /**
    * Set the FONT specification of this frame.
    * 
    * @param    font
    *           The font.
    */
   public void setFont(int font);

   /**
    * Set the STREAM-IO specification of this frame.
    * 
    * @param    streamIO
    *           <code>true</code> if the STREAM-IO specification is present.
    */
   public void setStreamIO(boolean streamIO);
}
